import * as jwt from "jsonwebtoken";
import { Get, JsonController, OnUndefined, Param } from 'routing-controllers';
import { OpenAPI, ResponseSchema } from 'routing-controllers-openapi';
import { getConnectionManager, Repository } from 'typeorm';
import { config } from '../config';
import { InvalidCredentialsError } from '../errors/AuthError';
import { RunnerNotFoundError } from '../errors/RunnerErrors';
import { Runner } from '../models/entities/Runner';
import { ResponseUser } from '../models/responses/ResponseUser';


@JsonController('/runners')
export class RunnerSelfServiceController {
	private runnerRepository: Repository<Runner>;

	/**
	 * Gets the repository of this controller's model/entity.
	 */
	constructor() {
		this.runnerRepository = getConnectionManager().get().getRepository(Runner);
	}

	@Get('/me/:jwt')
	@ResponseSchema(ResponseUser)
	@ResponseSchema(RunnerNotFoundError, { statusCode: 404 })
	@OnUndefined(RunnerNotFoundError)
	@OpenAPI({ description: 'Lists all information about yourself. <br> Please provide your runner jwt for auth.' })
	async get(@Param('jwt') token: string) {
		//TODO
	}

	/**
	 * Get's a runner by a provided jwt token.
	 * @param token The runner jwt provided by the runner to identitfy themselves.
	 */
	private async getRunner(token: string): Promise<Runner> {
		let jwtPayload = undefined
		try {
			jwtPayload = <any>jwt.verify(token, config.jwt_secret);
		} catch (error) {
			throw new InvalidCredentialsError();
		}

		console.log(jwtPayload);

		const runner = await this.runnerRepository.findOne({ id: jwtPayload["id"] });
		if (!runner) { throw new RunnerNotFoundError() }
		return runner;
	}

}